@@ -65,10 +65,17 @@ Numeric = TypeVar('Numeric', int, float, Decimal)
 
 
 def augment_override_cpp(fn):
+    """This Python function should replace the C++ implementation, if there is one."""
     fn._augment_override_cpp = True
     return fn
 
 
+def augment_if_no_cpp(fn):
+    """This Python function provides a Python implementation if no C++ implementation exists."""
+    fn._augment_if_no_cpp = True
+    return fn
+
+
 def _is_inherited_method(meth):
     # Augmenting a C++ with a method that cls inherits from the Python
     # object is never what we want.
@@ -140,12 +147,19 @@ def augments(cls_cpp: Type[Any]):
                 and name not in OVERRIDE_WHITELIST
                 and not getattr(getattr(cls, name), '_augment_override_cpp', False)
             ):
+                if getattr(getattr(cls, name), '_augment_if_no_cpp', False):
+                    # If tagged as "augment if no C++", we are supported older pybind11
+                    # versions that may not define some functions. If pybind11 provides
+                    # the required function, don't override it.
+                    continue
+
                 # If the original C++ class and Python support class both define the
                 # same name, we generally have a conflict, because this is augmentation
                 # not inheritance. However, if the method provided by the support class
                 # is an abstract method, then we can consider the C++ version the
                 # implementation. Also, pybind11 provides defaults for __eq__,
                 # __hash__ and __repr__ that we often do want to override directly.
+
                 raise RuntimeError(  # pragma: no cover
                     f"C++ {cls_cpp} and Python {cls} both define the same "
                     f"non-abstract method {name}: "
@@ -191,8 +205,7 @@ def _mudraw(buffer, fmt) -> bytes:
 
         proc = run(
             ['mudraw', '-F', fmt, '-o', '-', tmp_in.name],
-            stdout=PIPE,
-            stderr=PIPE,
+            capture_output=True,
             check=True,
         )
         return proc.stdout
@@ -928,9 +941,11 @@ class Extend_ObjectMapping:
         except KeyError:
             return default
 
+    @augment_if_no_cpp  # Remove this shim when dropping pybind11 < 2.8.0 support
     def keys(self):
         return KeysView(self)
 
+    @augment_if_no_cpp  # Remove this shim when dropping pybind11 < 2.8.0 support
     def values(self):
         return (v for _k, v in self.items())
 
@@ -1067,7 +1082,7 @@ class Extend_Page:
         return name
 
     def _over_underlay(
-        self, other, rect: Optional[Rectangle], under: bool = True
+        self, other, rect: Optional[Rectangle], under: bool, push_stack: bool
     ) -> None:
         formx = None
         if isinstance(other, Page):
@@ -1092,9 +1107,20 @@ class Extend_Page:
         formx_placed_name = self.add_resource(formx, Name.XObject)
         cs = self.calc_form_xobject_placement(formx, formx_placed_name, rect)
 
-        self.contents_add(cs, prepend=under)
+        if push_stack:
+            self.contents_add(b'q\n', prepend=True)  # prepend q
+            self.contents_add(b'Q\n', prepend=False)  # i.e. append Q
 
-    def add_overlay(self, other: Union[Object, Page], rect: Optional[Rectangle] = None):
+        self.contents_add(cs, prepend=under)
+        self.contents_coalesce()
+
+    def add_overlay(
+        self,
+        other: Union[Object, Page],
+        rect: Optional[Rectangle] = None,
+        *,
+        push_stack: Optional[bool] = True,
+    ):
         """Overlay another object on this page.
 
         Overlays will be drawn after all previous content, potentially drawing on top
@@ -1105,10 +1131,19 @@ class Extend_Page:
                 page.
             rect: The PDF rectangle (in PDF units) in which to draw the overlay.
                 If omitted, this page's trimbox, cropbox or mediabox will be used.
+            push_stack: If True (default), push the graphics stack of the existing
+                content stream to ensure that the overlay is rendered correctly.
+                Officially PDF limits the graphics stack depth to 32. Most
+                viewers will tolerate more, but excessive pushes may cause problems.
+                Multiple content streams may also be coalseced into a single content
+                stream where this parameter is True, since the PDF specification
+                permits PDF writers to coalesce streams as they see fit.
 
         .. versionadded:: 2.14
+
+        .. versionchanged:: 3.3.0
         """
-        return self._over_underlay(other, rect, under=False)
+        return self._over_underlay(other, rect, under=False, push_stack=push_stack)
 
     def add_underlay(
         self, other: Union[Object, Page], rect: Optional[Rectangle] = None
@@ -1126,7 +1161,7 @@ class Extend_Page:
 
         .. versionadded:: 2.14
         """
-        return self._over_underlay(other, rect, under=True)
+        return self._over_underlay(other, rect, under=True, push_stack=False)
 
     def contents_add(self, contents: Union[Stream, bytes], *, prepend: bool = False):
         """Append or prepend to an existing page's content stream.
@@ -1253,6 +1288,9 @@ class Extend_AttachedFileSpec:
         This function will automatically add a creation and modified date
         using the file system, and a MIME type inferred from the file's extension.
 
+        If the data required for the attach is in memory, use
+        :meth:`pikepdf.AttachedFileSpec` instead.
+
         Args:
             pdf: The Pdf to attach this file specification to.
             path: A file path for the file to attach to this Pdf.
@@ -1270,7 +1308,7 @@ class Extend_AttachedFileSpec:
             pdf,
             path.read_bytes(),
             description=description,
-            filename=str(path),
+            filename=str(path.name),
             mime_type=mime,
             creation_date=encode_pdf_date(
                 datetime.datetime.fromtimestamp(stat.st_ctime)
